/////////////////////////////////////////////////////////
// toon.fx - cartoon shader
/////////////////////////////////////////////////////////

#include <transform>
#include <fog>
#include <pos>
#include <normal>
#include <tangent>
#define INDOOR
#ifdef INDOOR
	#include <light>
#else
	float4 vecSunDir;
	float4 vecSunColor;
#endif
float4 vecLight;

const float facAmbient = 1.0; // factors to play with
const float facShading = 1.5;
const float facWidth = 2.0;   // line width

texture entSkin1;	// texture
texture mtlSkin1;	// toonlook
sampler sBaseTex = sampler_state { Texture = <entSkin1>; };
sampler sToon = sampler_state { 	Texture = <mtlSkin1>;};

struct toonOut
{
	float4 Pos:    POSITION;
	float  Fog:		FOG;
	float4 Color:	COLOR;
	float4 Ambient: COLOR1;
	float2 Tex:   	TEXCOORD0;
	float2 Bump:   TEXCOORD1;
	float3 Normal:	TEXCOORD2;
	float3 Light:	TEXCOORD3;
};

toonOut toonVS(
in float4 inPos: 	POSITION, 
in float3 inNormal:	NORMAL,
in float2 inTex: 	TEXCOORD0,
in float3 inTangent: TEXCOORD1)
{
	toonOut Out;
	Out.Pos	= DoTransform(inPos);
	Out.Tex	= inTex;
	Out.Fog	= DoFog(inPos);
	Out.Ambient = facAmbient*vecLight;
			
	CreateTangents(inNormal,inTangent);
	float3 P = DoPos(inPos);
			
	float3 lightVec;
// transform the output values into the 0..1 range
#ifdef INDOOR
	lightVec = normalize(vecLightPos[0].xyz - P);
	Out.Color = vecLightColor[0];
#else
	lightVec = -vecSunDir;
	Out.Color = vecSunColor;
#endif
	Out.Light = DoTangent(lightVec) * 0.5 + 0.5;
	Out.Bump = max(dot(DoNormal(inNormal), lightVec), 0);
	Out.Normal = DoTangent(DoPos(inNormal)) * 0.5 + 0.5;

	return Out;		
}


float4 toonPS(toonOut In): COLOR
{
	float4 base = tex2D(sBaseTex,In.Tex);
	float3 toonlook = tex2D(sToon,In.Bump);
	float diffuse = saturate(dot(In.Light*2 - 1,toonlook*2 - 1));
	diffuse *= saturate(4 * dot(In.Light*2 - 1,In.Normal*2 - 1));
	return base * (facShading * diffuse * In.Color + In.Ambient);
}

technique toon
{
	pass one
	{		
		VertexShader = compile vs_1_1 toonVS();
		PixelShader = compile ps_1_1 toonPS();
	}
	pass two 
	{
		CULLMODE=CW;
		vertexShaderConstant[0] = <matWorldViewProj>;
		vertexShaderConstant[16] = <facWidth>;
		
		VertexShader =	asm
		{
			vs_1_0
			dcl_position v0
			dcl_normal v3
			dcl_texcoord v7
			mov r0,v0
			mul r1,c16.x,v3
// Scale the normal
			add r0.xyz,r0.xyz,r1.xyz
			m4x4 oPos,r0,c0
// Transorm position to clip space
			mov oD0, c0
			mov r0,c0
		};
	}
}

technique fallback { pass one { } }
